@@ -15,6 +15,8 @@ from .sampling import (
     mirrored_sampling,
     sobol_sampling,
     halton_sampling,
+    Sobol, 
+    Halton
 )
 
 
@@ -25,6 +27,8 @@ class Parameters(AnnotatedStruct):
     ----------
     d: int
         The dimensionality of the problem
+    x0: np.ndarray
+        Initial guess of the population center of mass.
     target: float = -float("inf")
         The absolute target of the optimization problem
     budget: int = None
@@ -37,7 +41,7 @@ class Parameters(AnnotatedStruct):
         The number of offspring in the population
     mu: int = None
         The number of parents in the population
-    init_sigma: float = .5
+    sigma0: float = .5
         The initial value of sigma (step size)
     a_tpa: float = .5
         Parameter used in TPA
@@ -114,7 +118,7 @@ class Parameters(AnnotatedStruct):
             Kraków, Poland, September 11-15, 2010, PrCMAESart I, pages
             11–21, Berlin, Heidelberg, 2010. SpringerCMAESelberg.
             ACM, 2014.
-    weights_option: str = ('default', '1/mu', '1/2^mu', )
+    weights_option: str = ("default", "equal", "1/2^lambda", )
         Denoting the recombination weigths to be used.
             [7] Sander van Rijn, Hao Wang, Matthijs van Leeuwen, and Thomas Bäck. 2016.
             Evolving the Structure of Evolution Strategies. Computer 49, 5 (May 2016), 54–63.
@@ -236,12 +240,13 @@ class Parameters(AnnotatedStruct):
     """
 
     d: int
+    x0: np.ndarray = None
     target: float = -float("inf")
     budget: int = None
     n_generations: int = None
     lambda_: int = None
     mu: int = None
-    init_sigma: float = 0.5
+    sigma0: float = 0.2
     a_tpa: float = 0.5
     b_tpa: float = 0.0
     cs: float = None
@@ -277,6 +282,10 @@ class Parameters(AnnotatedStruct):
     ps_factor: float = 1.0
     compute_termination_criteria: bool = False
     sample_sigma: bool = False  # TODO make this a module
+    vectorized_fitness: bool = False
+    sobol: TypeVar("Sobol") = None
+    halton: TypeVar("Halton") = None
+
     __modules__ = (
         "active",
         "elitist",
@@ -309,11 +318,14 @@ class Parameters(AnnotatedStruct):
             a sampler
 
         """
-        sampler = {
-            "gaussian": gaussian_sampling,
-            "sobol": sobol_sampling,
-            "halton": halton_sampling,
-        }.get(self.base_sampler, gaussian_sampling)(self.d)
+        if self.base_sampler == 'gaussian':
+            sampler = gaussian_sampling(self.d)
+        elif self.base_sampler == 'sobol':
+            self.sobol = self.sobol or Sobol(self.d)
+            sampler = sobol_sampling(self.sobol)      
+        elif self.base_sampler == 'halton':
+            self.halton = self.halton or Halton(self.d)
+            sampler = halton_sampling(self.halton)
 
         if self.orthogonal:
             n_samples = max(
@@ -342,10 +354,14 @@ class Parameters(AnnotatedStruct):
         self.median_fitnesses = []
         self.best_fitnesses = []
         self.flat_fitnesses = deque(maxlen=self.d)
-        self.restarts = []
+        self.restarts = [0]
         self.bipop_parameters = BIPOPParameters(
             self.lambda_, self.budget, self.mu / self.lambda_
         )
+        self.chiN = self.d ** 0.5 * (1 - 1 / (4 * self.d) + 1 / (21 * self.d ** 2))
+        self.ds = 2 - (2 / self.d)
+        self.beta = np.log(2) / max((np.sqrt(self.d) * np.log(self.d)), 1)
+        self.succes_ratio = .25
 
     def init_selection_parameters(self) -> None:
         """Initialization function for parameters that influence in selection."""
@@ -377,8 +393,7 @@ class Parameters(AnnotatedStruct):
 
         TODO: check if we can move this to separate object.
         """
-        if len(self.restarts) == 0:
-            self.restarts.append(self.t)
+       
         self.max_iter = 100 + 50 * (self.d + 3) ** 2 / np.sqrt(self.lambda_)
         self.nbin = 10 + int(np.ceil(30 * self.d / self.lambda_))
         self.n_stagnation = min(int(120 + (30 * self.d / self.lambda_)), 20000)
@@ -391,22 +406,22 @@ class Parameters(AnnotatedStruct):
 
         Examples are recombination weights and learning rates for the covariance
         matrix adapation.
-        TODO: clean the initlialization of these weights, this can be wrong in some cases
-        when mu != .5
         """
         if self.weights_option == "equal":
-            ws = np.ones(self.lambda_) / self.lambda_
-            self.weights = np.append(ws[:self.mu], ws[self.mu::-1] * -1)
-
-            if self.lambda_ % 2 != 0:
-                self.weights = np.append([1 / self.lambda_], self.weights[:-1])
-        elif self.weights_option == "1/2^lambda":
-            ws = 1 / 2 ** np.arange(1, self.lambda_ + 1) + (
-                (1 / (2 ** self.lambda_)) / self.lambda_
+            ws = 1 / self.mu
+            self.weights = np.append(
+                np.ones(self.mu) * ws, np.ones(self.lambda_ - self.mu) * ws * -1
             )
-            self.weights = np.append(ws[:self.mu], ws[self.mu::-1] * -1)
-            if self.lambda_ % 2 != 0:
-                self.weights = np.append([1 / self.lambda_ ** 2], self.weights[:-1])
+        elif self.weights_option == "1/2^lambda":
+            base = np.float64(2)
+            positive = self.mu / (base ** np.arange(1, self.mu + 1)) + ( # "self.mu /" should be "1 /"
+                (1 / (base ** self.mu)) / self.mu
+            )
+            n = self.lambda_ - self.mu
+            negative = (1 / (base ** np.arange(1, n + 1)) + (
+                (1 / (base ** n)) / n
+            ))[::-1] * -1
+            self.weights = np.append(positive, negative)
         else:
             self.weights = np.log((self.lambda_ + 1) / 2) - np.log(
                 np.arange(1, self.lambda_ + 1)
@@ -414,16 +429,16 @@ class Parameters(AnnotatedStruct):
 
         self.pweights = self.weights[: self.mu]
         self.nweights = self.weights[self.mu:]
-
         self.mueff = self.pweights.sum() ** 2 / (self.pweights ** 2).sum()
         mueff_neg = self.nweights.sum() ** 2 / (self.nweights ** 2).sum()
+
+        self.pweights = self.pweights / self.pweights.sum()
         self.c1 = self.c1 or 2 / ((self.d + 1.3) ** 2 + self.mueff)
         self.cmu = self.cmu or min(1 - self.c1, (2 * (
             (self.mueff - 2 + (1 / self.mueff))
             / ((self.d + 2) ** 2 + (2 * self.mueff / 2))
         )))
 
-        self.pweights = self.pweights / self.pweights.sum()
         amu_neg = 1 + (self.c1 / self.mu)
         amueff_neg = 1 + ((2 * mueff_neg) / (self.mueff + 2))
         aposdef_neg = (1 - self.c1 - self.cmu) / (self.d * self.cmu)
@@ -449,11 +464,6 @@ class Parameters(AnnotatedStruct):
         self.damps = 1.0 + (
             2.0 * max(0.0, np.sqrt((self.mueff - 1) / (self.d + 1)) - 1) + self.cs
         )
-        self.chiN = self.d ** 0.5 * (1 - 1 / (4 * self.d) + 1 / (21 * self.d ** 2))
-        self.ds = 2 - (2 / self.d)
-
-        self.beta = np.log(2) / max((np.sqrt(self.d) * np.log(self.d)), 1)
-        self.succes_ratio = .25
 
     def init_dynamic_parameters(self) -> None:
         """Initialization function of parameters that represent the dynamic state of the CMA-ES.
@@ -461,16 +471,19 @@ class Parameters(AnnotatedStruct):
         Examples of such parameters are the Covariance matrix C and its 
         eigenvectors and the learning rate sigma.
         """
-        self.sigma = self.init_sigma
-        self.m = np.random.rand(self.d, 1)
-        self.m_old = np.empty((self.d, 1))
-        self.dm = np.zeros(self.d)
-        self.pc = np.zeros((self.d, 1))
-        self.ps = np.zeros((self.d, 1))
-        self.B = np.eye(self.d)
-        self.C = np.eye(self.d)
-        self.D = np.ones((self.d, 1))
-        self.inv_root_C = np.eye(self.d)
+        self.sigma = np.float64(self.sigma0) * (self.ub[0,0] - self.lb[0,0])
+        if hasattr(self, "m") or self.x0 is None: 
+            self.m = np.float64(np.random.uniform(self.lb, self.ub, (self.d, 1)))
+        else:
+            self.m = np.float64(self.x0.copy())
+        self.m_old = np.empty((self.d, 1), dtype=np.float64)
+        self.dm = np.zeros(self.d, dtype=np.float64)
+        self.pc = np.zeros((self.d, 1), dtype=np.float64)
+        self.ps = np.zeros((self.d, 1), dtype=np.float64)
+        self.B = np.eye(self.d, dtype=np.float64)
+        self.C = np.eye(self.d, dtype=np.float64)
+        self.D = np.ones((self.d, 1), dtype=np.float64)
+        self.inv_root_C = np.eye(self.d, dtype=np.float64)
         self.s = 0
         self.rank_tpa = None
         self.hs = True
@@ -572,6 +585,7 @@ class Parameters(AnnotatedStruct):
                     ), 2
                 )
             )
+
             rank_mu = self.cmu * (weights * self.population.y @ self.population.y.T)
         else:
             rank_mu = self.cmu * (
@@ -595,9 +609,13 @@ class Parameters(AnnotatedStruct):
             self.init_dynamic_parameters()
         else:
             self.C = np.triu(self.C) + np.triu(self.C, 1).T
+
             self.D, self.B = linalg.eigh(self.C)
-            self.D = np.sqrt(self.D.astype(complex).reshape(-1, 1)).real
-            self.inv_root_C = np.dot(self.B, self.D ** -1 * self.B.T)
+            if np.all(self.D > 0):
+                self.D = np.sqrt(self.D.reshape(-1, 1))
+                self.inv_root_C = np.dot(self.B, self.D ** -1 * self.B.T)
+            else:
+                self.init_dynamic_parameters()
 
     def adapt_evolution_paths(self) -> None:
         """Method to adapt the evolution paths ps and pc."""
@@ -618,7 +636,11 @@ class Parameters(AnnotatedStruct):
     def perform_local_restart(self) -> None:
         """Method performing local restart, if a restart strategy is specified."""
         if self.local_restart:
-            if self.local_restart == "IPOP":
+
+            if len(self.restarts) == 0:
+                self.restarts.append(self.t)
+
+            if self.local_restart == "IPOP" and self.mu < 512: 
                 self.mu *= self.ipop_factor
                 self.lambda_ *= self.ipop_factor
 
@@ -715,6 +737,7 @@ class Parameters(AnnotatedStruct):
                 raise AttributeError(
                     f"{filename} does not contain " "a Parameters object"
                 )
+        np.random.set_state(parameters.random_state)
         parameters.sampler = parameters.get_sampler()
         return parameters
 
@@ -727,12 +750,17 @@ class Parameters(AnnotatedStruct):
             The name of the file to save to.
 
         """
+        sampler = self.sampler 
         with open(filename, "wb") as f:
             self.sampler = None
+            self.random_state = np.random.get_state()
             pickle.dump(self, f)
+        self.sampler = sampler
+
 
     def record_statistics(self) -> None:
         """Method for recording metadata."""
+        # if self.local_restart or self.compute_termination_criteria:
         self.flat_fitnesses.append(
             self.population.f[0] == self.population.f[self.flat_fitness_index]
         )
@@ -751,27 +779,27 @@ class Parameters(AnnotatedStruct):
         if self.local_restart or self.compute_termination_criteria:
             _t = self.t % self.d
             diag_C = np.diag(self.C.T)
-            d_sigma = self.sigma / self.init_sigma
+            d_sigma = self.sigma / self.sigma0
             best_fopts = self.best_fitnesses[self.last_restart:]
             median_fitnesses = self.median_fitnesses[self.last_restart:]
-
+            time_since_restart = self.t - self.last_restart
             self.termination_criteria = (
                 dict()
                 if self.lambda_ > self.max_lambda_
                 else {
-                    "max_iter": (self.t - self.last_restart > self.max_iter),
+                    "max_iter": (time_since_restart > self.max_iter),
                     "equalfunvalues": (
                         len(best_fopts) > self.nbin
                         and np.ptp(best_fopts[-self.nbin:]) == 0
                     ),
                     "flat_fitness": (
-                        self.t - self.last_restart > self.flat_fitnesses.maxlen
+                        time_since_restart > self.flat_fitnesses.maxlen
                         and len(self.flat_fitnesses) == self.flat_fitnesses.maxlen
                         and np.sum(self.flat_fitnesses) > (self.d / 3)
                     ),
                     "tolx": np.all(
                         (np.append(self.pc.T, diag_C) * d_sigma)
-                        < (self.tolx * self.init_sigma)
+                        < (self.tolx * self.sigma0)
                     ),
                     "tolupsigma": (d_sigma > self.tolup_sigma * np.sqrt(self.D.max())),
                     "conditioncov": np.linalg.cond(self.C) > self.condition_cov,
@@ -786,12 +814,12 @@ class Parameters(AnnotatedStruct):
                         (0.2 * self.sigma * np.sqrt(diag_C) + self.m) == self.m
                     ),
                     "stagnation": (
-                        self.t - self.last_restart > self.n_stagnation
+                        time_since_restart > self.n_stagnation
                         and (
-                            np.median(best_fopts[-int(0.3 * self.t):])
-                            >= np.median(best_fopts[: int(0.3 * self.t)])
-                            and np.median(median_fitnesses[-int(0.3 * self.t):])
-                            >= np.median(median_fitnesses[: int(0.3 * self.t)])
+                            np.median(best_fopts[-int(0.3 * time_since_restart):])
+                            >= np.median(best_fopts[: int(0.3 * time_since_restart)])
+                            and np.median(median_fitnesses[-int(0.3 * time_since_restart):])
+                            >= np.median(median_fitnesses[: int(0.3 * time_since_restart)])
                         )
                     ),
                 }
@@ -831,6 +859,17 @@ class Parameters(AnnotatedStruct):
         self.init_selection_parameters()
         self.init_adaptation_parameters()
         self.init_local_restart_parameters()
+        
+    def update_popsize(self, lambda_new):
+        """Manually control the population size."""
+        if self.local_restart is not None:
+            warnings.warn("Modification of population size is disabled when local restart startegies are used")
+            return
+        self.lambda_ = lambda_new
+        self.mu = lambda_new//2
+        self.init_selection_parameters()
+        self.init_adaptation_parameters()
+        self.init_local_restart_parameters()
 
 
 class BIPOPParameters(AnnotatedStruct):
