@@ -1,5 +1,6 @@
 """Module containing tests for ModularCMA-ES Parameters."""
 
+from email.mime import base
 import os
 import unittest
 import warnings
@@ -54,7 +55,7 @@ class TestParameters(unittest.TestCase):
         self.p.used_budget += 11
         self.p.bipop_parameters.adapt(self.p.used_budget)
         self.assertEqual(self.p.bipop_parameters.large, True)
-        bp = self.p.bipop_parameters
+        bp = self.p.bipop_parameters 
         self.assertEqual(bp.lambda_, self.p.lambda_ * 2)
         self.assertEqual(bp.mu, self.p.mu * 2)
         self.assertEqual(bp.sigma, 2)
@@ -63,7 +64,7 @@ class TestParameters(unittest.TestCase):
         self.assertEqual(self.p.bipop_parameters.large, False)
         self.assertLessEqual(bp.lambda_, self.p.lambda_)
         self.assertLessEqual(bp.mu, self.p.mu)
-        self.assertLessEqual(bp.sigma, self.p.init_sigma)
+        self.assertLessEqual(bp.sigma, self.p.sigma0)
         self.p.used_budget += 11
         bp.adapt(self.p.used_budget)
         self.assertEqual(bp.used_budget, 33)
@@ -177,6 +178,17 @@ class TestParameters(unittest.TestCase):
             self.p.load(tmpfile)
         os.remove(tmpfile)
 
+    def test_save_load_samplers(self):
+        tmpfile = os.path.join(os.path.dirname(__file__), "tmp.pkl")
+        for base_sampler in getattr(Parameters, "base_sampler").options:
+            p = Parameters(2, base_sampler=base_sampler)
+            p.save(tmpfile)
+            sample = next(p.sampler)
+            p2 = Parameters.load(tmpfile)
+            sample2 = next(p2.sampler)
+            self.assertTrue(np.all(sample == sample2))
+        os.remove(tmpfile)
+
     def test_fix_lambda_even(self):
         self.p.lambda_ = 11
         self.p.mirrored = 'mirrored pairwise'
